# -*- coding: utf-8 -*-
from DateTime import DateTime
from plone import api
from Products.CMFCore.utils import getToolByName
from Products.Five.browser import BrowserView
from rer.bandi import _
from six.moves.urllib.parse import quote
from zope.component import getUtility
from zope.i18n import translate
from zope.schema.interfaces import IVocabularyFactory


class SearchBandiForm(BrowserView):
    def getUniqueValuesForIndex(self, index):
        """
        get uniqueValuesFor a given index
        """
        pc = api.portal.get_tool(name='portal_catalog')
        return pc.uniqueValuesFor(index)

    def getVocabularyTermsForForm(self, vocab_name):
        """
        Return the values of destinatari vocabulary
        """
        dest_utility = getUtility(IVocabularyFactory, vocab_name)

        dest_values = []

        dest_vocab = dest_utility(self.context)

        for dest in dest_vocab:
            if dest.title != u'select_label':
                dest_values.append(dest.value)
        return dest_values


class SearchBandi(BrowserView):
    """
    A view for search bandi results
    """

    def searchBandi(self):
        """
        return a list of bandi
        """
        pc = getToolByName(self.context, "portal_catalog")
        stato = self.request.form.get("stato_bandi", "")
        SearchableText = self.request.form.get("SearchableText", "")
        query = self.request.form.copy()
        if stato:
            now = DateTime()
            if stato == "open":
                query["scadenza_bando"] = {"query": now, "range": "min"}
                query["chiusura_procedimento_bando"] = {
                    "query": now,
                    "range": "min",
                }
            if stato == "inProgress":
                query["scadenza_bando"] = {"query": now, "range": "max"}
                query["chiusura_procedimento_bando"] = {
                    "query": now,
                    "range": "min",
                }
            if stato == "closed":
                query["chiusura_procedimento_bando"] = {
                    "query": now,
                    "range": "max",
                }
        if "SearchableText" in self.request.form and not SearchableText:
            del query["SearchableText"]
        return pc(**query)

    @property
    def rss_query(self):
        """
        set rss query with the right date
        """
        query = self.request.QUERY_STRING
        stato = self.request.form.get("stato_bandi", "")
        if stato:
            now = DateTime().ISO()
            if stato == "open":
                query = (
                    query
                    + "&scadenza_bando.query:record=%s&scadenza_bando.range:record=min"
                    % quote(now)
                )
                query = (
                    query
                    + "&chiusura_procedimento_bando.query:record=%s&chiusura_procedimento_bando.range:record=min"
                    % quote(now)
                )
            if stato == "inProgress":
                query = (
                    query
                    + "&amp;scadenza_bando.query:record=%s&scadenza_bando.range:record=max"
                    % quote(now)
                )
                query = (
                    query
                    + "&amp;chiusura_procedimento_bando.query:record=%s&chiusura_procedimento_bando.range:record=min"
                    % quote(now)
                )
            if stato == "closed":
                query = (
                    query
                    + "&amp;chiusura_procedimento_bando.query:record=%s&chiusura_procedimento_bando.range:record=max"
                    % quote(now)
                )

        return query

    def getBandoState(self, bando):
        """
        """

        scadenza_bando = bando.scadenza_bando
        chiusura_procedimento_bando = bando.chiusura_procedimento_bando
        state = ("open", translate(_(u"Open"), context=self.request))
        if scadenza_bando and scadenza_bando.isPast():
            if (
                chiusura_procedimento_bando
                and chiusura_procedimento_bando.isPast()
            ):
                state = (
                    "closed",
                    translate(_(u"Closed"), context=self.request),
                )
            else:
                state = (
                    "inProgress",
                    translate(_(u"In progress"), context=self.request),
                )
        else:
            if (
                chiusura_procedimento_bando
                and chiusura_procedimento_bando.isPast()
            ):
                state = (
                    "closed",
                    translate(_(u"Closed"), context=self.request),
                )

        return state

    def isValidDeadline(self, date):
        """
        """

        if not date:
            return False
        if date.Date() == "2100/12/31":
            # a default date for bandi that don't have a defined deadline
            return False
        return True

    def getSearchResultsDescriptionLength(self):
        length = api.portal.get_registry_record(
            "plone.search_results_description_length"
        )
        return length

    def getAllowAnonymousViewAbout(self):
        return api.portal.get_registry_record("plone.allow_anon_views_about")

    def getTypesUseViewActionInListings(self):

        return api.portal.get_registry_record(
            "plone.types_use_view_action_in_listings"
        )



# -*- coding: utf-8 -*-
from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_FIXTURE
from plone.app.robotframework.testing import REMOTE_LIBRARY_BUNDLE_FIXTURE
from plone.app.testing import applyProfile
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PloneSandboxLayer
from plone.restapi.testing import PloneRestApiDXLayer
from plone.testing import z2

import rer.bandi


class RerBandiLayer(PloneSandboxLayer):

    defaultBases = (PLONE_APP_CONTENTTYPES_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        # Load any other ZCML that is required for your tests.
        # The z3c.autoinclude feature is disabled in the Plone fixture base
        # layer.
        self.loadZCML(package=rer.bandi)

    def setUpPloneSite(self, portal):
        applyProfile(portal, 'rer.bandi:default')


RER_BANDI_FIXTURE = RerBandiLayer()


RER_BANDI_INTEGRATION_TESTING = IntegrationTesting(
    bases=(RER_BANDI_FIXTURE,), name='RerBandiLayer:IntegrationTesting'
)


RER_BANDI_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(RER_BANDI_FIXTURE,), name='RerBandiLayer:FunctionalTesting'
)


RER_BANDI_ACCEPTANCE_TESTING = FunctionalTesting(
    bases=(
        RER_BANDI_FIXTURE,
        REMOTE_LIBRARY_BUNDLE_FIXTURE,
        z2.ZSERVER_FIXTURE,
    ),
    name='RerBandiLayer:AcceptanceTesting',
)


class RerBandiRestApiLayer(PloneRestApiDXLayer):

    defaultBases = (PLONE_APP_CONTENTTYPES_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        super(RerBandiRestApiLayer, self).setUpZope(app, configurationContext)

        self.loadZCML(package=rer.bandi)

    def setUpPloneSite(self, portal):
        super(RerBandiRestApiLayer, self).setUpPloneSite(portal)
        applyProfile(portal, 'rer.bandi:default')


RER_BANDI_API_FIXTURE = RerBandiRestApiLayer()
RER_BANDI_API_INTEGRATION_TESTING = IntegrationTesting(
    bases=(RER_BANDI_API_FIXTURE,), name="RerBandiRestApiLayer:Integration"
)

RER_BANDI_API_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(RER_BANDI_API_FIXTURE, z2.ZSERVER_FIXTURE),
    name="RerBandiRestApiLayer:Functional",
)



# -*- coding: utf-8 -*-
from plone import api
from datetime import datetime
from datetime import timedelta
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.restapi.testing import RelativeSession
from Products.CMFCore.utils import getToolByName
from rer.bandi.testing import RER_BANDI_API_FUNCTIONAL_TESTING


import requests
import transaction
import unittest


class SearchBandiTest(unittest.TestCase):

    layer = RER_BANDI_API_FUNCTIONAL_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.absolute_url()
        self.catalog = getToolByName(self.portal, "portal_catalog")

        self.portal = self.layer["portal"]
        self.request = self.layer["request"]

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)

        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        # Object creation
        self.bando = api.content.create(
            container=self.portal, type="Bando", title="Bando foo setup"
        )
        self.document = api.content.create(
            container=self.portal, type="Document", title="Empty page"
        )

        transaction.commit()

    def test_route_exists(self):
        response = self.api_session.get("/@search_bandi_rest")

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.headers.get("Content-Type"), "application/json")

    def test_search_only_bandi(self):
        response = self.api_session.get("/@search_bandi_rest")

        results = response.json()
        self.assertEqual(
            results[u"items_total"], 1,
        )

    def test_search_by_stato_bandi_closed(self):
        menouno = timedelta(days=-1)
        menocinque = timedelta(days=-5)

        self.bando = api.content.create(
            container=self.portal,
            type="Bando",
            title="Bando expired",
            scadenza_bando=datetime.now() + menocinque,
            chiusura_procedimento_bando=datetime.now() + menouno,
        )
        self.bando = api.content.create(
            container=self.portal, type="Bando", title="Bando foo 1"
        )
        transaction.commit()

        response = self.api_session.get("/@search_bandi_rest?stato_bandi=closed")
        results = response.json()

        self.assertEqual(
            results[u"items_total"], 1,
        )

    def test_search_by_stato_bandi_inprogress(self):
        menocinque = timedelta(days=-5)
        piucinque = timedelta(days=+5)

        self.bando = api.content.create(
            container=self.portal,
            type="Bando",
            title="Bando in progress",
            scadenza_bando=datetime.now() + menocinque,
            chiusura_procedimento_bando=datetime.now() + piucinque,
        )
        self.bando = api.content.create(
            container=self.portal, type="Bando", title="Bando foo 2"
        )
        transaction.commit()

        response = self.api_session.get("/@search_bandi_rest?stato_bandi=inProgress")
        results = response.json()

        self.assertEqual(
            results[u"items_total"], 1,
        )

    def test_search_by_stato_bandi_open(self):
        piuuno = timedelta(days=+1)
        piucinque = timedelta(days=+5)

        self.bando = api.content.create(
            container=self.portal,
            type="Bando",
            title="Bando open",
            scadenza_bando=datetime.now() + piuuno,
            chiusura_procedimento_bando=datetime.now() + piucinque,
        )
        self.bando = api.content.create(
            container=self.portal, type="Bando", title="Bando foo 3"
        )
        transaction.commit()

        response = self.api_session.get("/@search_bandi_rest?stato_bandi=open")
        results = response.json()

        bandi = results['items']
        trovato = [x['title'] for x in bandi if x['title'] == u'Bando open']

        self.assertEqual(
            len(trovato), 1,
        )



# -*- coding: utf-8 -*-
from rer.bandi.testing import RER_BANDI_INTEGRATION_TESTING
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID

import unittest


class BandoViewTest(unittest.TestCase):

    layer = RER_BANDI_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.bando = api.content.create(
            container=self.portal, type='Bando', title='Bando foo'
        )

    def test_bando_views_registered(self):
        view = api.content.get_view(
            name='bando_view', context=self.bando, request=self.request
        )
        self.assertTrue(view.__name__ == 'bando_view')

        view_right = api.content.get_view(
            name='bando_right_view', context=self.bando, request=self.request
        )
        self.assertTrue(view_right.__name__ == 'bando_right_view')

    def test_destinatari_in_view(self):
        view = api.content.get_view(
            name='bando_view', context=self.bando, request=self.request
        )
        self.assertNotIn('Who can apply', view())

        bando_new = api.content.create(
            container=self.portal,
            type='Bando',
            title='Bando new',
            destinatari=['foo', 'bar'],
        )
        view_new = api.content.get_view(
            name='bando_view', context=bando_new, request=self.request
        )
        self.assertIn('Who can apply', view_new())
        self.assertIn('<li>foo</li>', view_new())
        self.assertIn('<li>bar</li>', view_new())

    def test_destinatari_in_right_view(self):
        view = api.content.get_view(
            name='bando_right_view', context=self.bando, request=self.request
        )
        self.assertNotIn('Who can apply', view())

        bando_new = api.content.create(
            container=self.portal,
            type='Bando',
            title='Bando new',
            destinatari=['foo', 'bar'],
        )
        view_new = api.content.get_view(
            name='bando_right_view', context=bando_new, request=self.request
        )
        self.assertIn('Who can apply', view_new())
        self.assertIn('<li>foo</li>', view_new())
        self.assertIn('<li>bar</li>', view_new())

    def test_tipologia_bando_in_view(self):
        view = api.content.get_view(
            name='bando_view', context=self.bando, request=self.request
        )

        # Because it's a required field
        self.assertIn('Announcement type', view())

        bando_new = api.content.create(
            container=self.portal,
            type='Bando',
            title='Bando new',
            tipologia_bando='Type a',
        )
        view_new = api.content.get_view(
            name='bando_view', context=bando_new, request=self.request
        )
        self.assertIn('Announcement type', view_new())
        self.assertIn('Type a', view_new())

    def test_tipologia_bando_in_right_view(self):
        view = api.content.get_view(
            name='bando_right_view', context=self.bando, request=self.request
        )

        # Because it's a required field
        self.assertIn('Announcement type', view())

        bando_new = api.content.create(
            container=self.portal,
            type='Bando',
            title='Bando new',
            tipologia_bando='Type a',
        )
        view_new = api.content.get_view(
            name='bando_right_view', context=bando_new, request=self.request
        )
        self.assertIn('Announcement type', view_new())
        self.assertIn('Type a', view_new())

    def test_finanziatori_in_view(self):
        view = api.content.get_view(
            name='bando_view', context=self.bando, request=self.request
        )
        self.assertNotIn('Financed by EU programmes', view())

        bando_new = api.content.create(
            container=self.portal,
            type='Bando',
            title='Bando new',
            finanziatori=['Found1', 'Found2'],
        )
        view_new = api.content.get_view(
            name='bando_view', context=bando_new, request=self.request
        )
        self.assertIn('Financed by EU programmes', view_new())
        self.assertIn('<span>Found1 | Found2</span>', view_new())

    def test_finanziatori_in_right_view(self):
        view = api.content.get_view(
            name='bando_right_view', context=self.bando, request=self.request
        )
        self.assertNotIn('Financed by EU programmes', view())

        bando_new = api.content.create(
            container=self.portal,
            type='Bando',
            title='Bando new',
            finanziatori=['Found1', 'Found2'],
        )
        view_new = api.content.get_view(
            name='bando_right_view', context=bando_new, request=self.request
        )
        self.assertIn('Financed by EU programmes', view_new())
        self.assertIn('<span>Found1 | Found2</span>', view_new())

    def test_materie_in_view(self):
        view = api.content.get_view(
            name='bando_view', context=self.bando, request=self.request
        )
        self.assertNotIn('Topic', view())

        bando_new = api.content.create(
            container=self.portal,
            type='Bando',
            title='Bando new',
            materie=['Topic1', 'Topic2'],
        )
        view_new = api.content.get_view(
            name='bando_view', context=bando_new, request=self.request
        )
        self.assertIn('Topic', view_new())
        self.assertIn('<span>Topic1 | Topic2</span>', view_new())

    def test_materie_in_right_view(self):
        view = api.content.get_view(
            name='bando_right_view', context=self.bando, request=self.request
        )
        self.assertNotIn('Topic', view())

        bando_new = api.content.create(
            container=self.portal,
            type='Bando',
            title='Bando new',
            materie=['Topic1', 'Topic2'],
        )
        view_new = api.content.get_view(
            name='bando_right_view', context=bando_new, request=self.request
        )
        self.assertIn('Topic', view_new())
        self.assertIn('<span>Topic1 | Topic2</span>', view_new())



# -*- coding: utf-8 -*-
from zope.interface import implementer
from rer.bandi.interfaces.bando import IBando
from plone.dexterity.content import Container


@implementer(IBando)
class Bando(Container):
    """ """



# -*- coding: utf8 -*-
from dateutil.parser import parse
from plone.app.querystring.interfaces import IParsedQueryIndexModifier
from zope.interface import implementer


import six


@implementer(IParsedQueryIndexModifier)
class BandiKeywordIndex(object):

    """
    All Keyword fields in Plone currently uses a utf-8 encoded string.
    When a catalog query tries to compare a unicode string from the
    parsedquery with existing utf-8 encoded string indexes unindexing
    will fail with a UnicodeDecodeError. To prevent this from happening
    we always encode the Subject query.

    XXX: As soon as Plone uses unicode for all indexes, this code can
    be removed.
    """

    def __call__(self, value):
        query = value['query']
        # query can be a unicode string or a list of unicode strings.
        if six.PY2 and isinstance(query, six.text_type):
            query = query.encode("utf-8")
        elif isinstance(query, list):
            # We do not want to change the collections' own query string,
            # therefore we create a new copy of the list.
            copy_of_query = list(query)
            # Iterate over all query items and encode them if they are
            # unicode strings
            i = 0
            for item in copy_of_query:
                if six.PY2 and isinstance(item, six.text_type):
                    copy_of_query[i] = item.encode("utf-8")
                i += 1
            query = copy_of_query
        else:
            pass
        value['query'] = query
        return (self.index_name, value)


class Destinatari(BandiKeywordIndex):

    index_name = 'destinatari'


class Tipologia(BandiKeywordIndex):

    index_name = 'tipologia_bando'


class Finanziatori(BandiKeywordIndex):

    index_name = 'finanziatori'


class Materie(BandiKeywordIndex):

    index_name = 'materie'



# -*- coding: utf-8 -*-
from plone.app.textfield import RichText
from plone.autoform import directives
from plone.autoform import directives as form
from plone.supermodel import model
from rer.bandi import _
from z3c.form.browser.checkbox import CheckBoxFieldWidget
from z3c.form.interfaces import NOVALUE
from zope import schema
from z3c.form.browser.radio import RadioFieldWidget
from zope.interface import Invalid
from zope.schema._messageid import _ as zsm


def checkRequiredField(value):
    if value == NOVALUE or value == []:
        raise Invalid(zsm("""Required input is missing."""))
    return True


class IBandoSchema(model.Schema):
    """ A Dexterity schema for Annoucements """

    directives.widget(tipologia_bando=RadioFieldWidget)
    tipologia_bando = schema.Choice(
        title=_('tipologia_bando_label', default=u"Announcement type"),
        description=_('tipologia_bando_help', default=''),
        vocabulary='rer.bandi.tipologie.vocabulary',
        required=True,
    )

    directives.widget(destinatari=CheckBoxFieldWidget)
    destinatari = schema.List(
        title=_('destinatari_label', default=u'Who can apply'),
        description=_(
            'bandi_multiselect_help', default='Select one or more values.'
        ),
        constraint=checkRequiredField,
        required=True,
        value_type=schema.Choice(
            vocabulary='rer.bandi.destinatari.vocabulary'
        ),
    )

    directives.widget(finanziatori=CheckBoxFieldWidget)
    finanziatori = schema.List(
        title=_('finanziatori_label', default=u'Financed by EU programmes'),
        description=_(
            'bandi_multiselect_help', default='Select one or more values.'
        ),
        required=False,
        value_type=schema.Choice(
            vocabulary='rer.bandi.finanziatori.vocabulary'
        ),
    )

    directives.widget(materie=CheckBoxFieldWidget)
    materie = schema.List(
        title=_('materie_label', default=u'Topic'),
        description=_(
            'bandi_multiselect_help', default='Select one or more values.'
        ),
        required=False,
        value_type=schema.Choice(vocabulary='rer.bandi.materie.vocabulary'),
    )

    scadenza_bando = schema.Datetime(
        title=_('scadenza_bando_label', default=u"Expiration date and time"),
        description=_(
            'scadenza_bando_help',
            default=u"Deadline to participate in the announcement",
        ),
        required=False,
    )
    chiusura_procedimento_bando = schema.Date(
        title=_(
            'chiusura_procedimento_bando_label',
            default=u"Closing date procedure",
        ),
        description=_('chiusura_procedimento_bando_help', default=u''),
        required=False,
    )

    riferimenti_bando = RichText(
        title=_('riferimenti_bando_label', default=u"References"),
        description=_('riferimenti_bando_help', default=u""),
        required=False,
    )

    form.order_after(tipologia_bando='IRichText.text')
    form.order_after(destinatari='tipologia_bando')
    form.order_after(finanziatori='destinatari')
    form.order_after(materie='finanziatori')
    form.order_after(scadenza_bando='materie')
    form.order_after(chiusura_procedimento_bando='scadenza_bando')
    form.order_after(riferimenti_bando='chiusura_procedimento_bando')



# -*- coding: utf-8 -*-
from plone import api
from plone.memoize import ram
from Products.Five import BrowserView
from time import time

import pkg_resources

JS_TEMPLATE = '{portal_url}/++plone++rer.bandi.static/dist/{env_mode}/{name}.js?v={version}'
CSS_TEMPLATE = '{portal_url}/++plone++rer.bandi.static/dist/{env_mode}/{name}.css?v={version}'


class View(BrowserView):
    """ """

    @ram.cache(lambda *args: time() // (60 * 60))
    def get_version(self):
        return pkg_resources.get_distribution('rer.bandi').version

    def get_env_mode(self):
        return (
            api.portal.get_registry_record('plone.resources.development')
            and 'dev'
            or 'prod'
        )

    def get_resource_js(self, name='main'):
        return JS_TEMPLATE.format(
            portal_url=api.portal.get().absolute_url(),
            env_mode=self.get_env_mode(),
            name=name,
            version=self.get_version(),
        )

    def get_resource_css(self, name='main'):
        return CSS_TEMPLATE.format(
            portal_url=api.portal.get().absolute_url(),
            env_mode=self.get_env_mode(),
            name=name,
            version=self.get_version(),
        )



# -*- coding: utf-8 -*-
from zope.interface import implementer
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleVocabulary, SimpleTerm

TIPOLOGIE_BANDO = [
    u'Agevolazioni, finanziamenti, contributi',
    u'Accreditamenti, albi, elenchi',
    u'Autorizzazioni di attività',
    u'Manifestazioni di interesse',
]

DESTINATARI_BANDO = [
    u'Cittadini',
    u'Confidi',
    u'Cooperative',
    u'Enti del Terzo settore',
    u'Enti e laboratori di ricerca',
    u'Enti pubblici',
    u'Grandi imprese',
    u'Liberi professionisti',
    u'Micro imprese',
    u'Partenariato pubblico/privato',
    u'PMI',
    u'Scuole, università, enti di formazione',
    u'Soggetti accreditati',
]

FINANZIATORI_BANDO = [u'FESR', u'FSE', u'FEASR', u'FEAMP']

MATERIE_BANDO = [
    u'Agricoltura e sviluppo delle aree rurali',
    u'Ambiente',
    u'Beni immobili e mobili',
    u'Cultura',
    u'Diritti e sociale',
    u'Edilizia e rigenerazione urbana',
    u'Energia',
    u'Estero',
    u'Fauna, caccia, pesca',
    u'Imprese e commercio',
    u'Innovazione e ICT',
    u'Istruzione e formazione',
    u'Lavoro',
    u'Mobilità e trasporti',
    u'Ricerca',
    u'Riordino istituzionale',
    u'Sport',
]


class BandiBaseVocabularyFactory(object):
    @property
    def terms(self):
        return [
            SimpleTerm(value=x, token=x.encode('utf-8'), title=x)
            for x in self.vocab_name
        ]

    def __call__(self, context):
        return SimpleVocabulary(self.terms)


@implementer(IVocabularyFactory)
class TipologieBandoVocabularyFactory(BandiBaseVocabularyFactory):

    vocab_name = TIPOLOGIE_BANDO


@implementer(IVocabularyFactory)
class DestinatariBandoVocabularyFactory(BandiBaseVocabularyFactory):

    vocab_name = DESTINATARI_BANDO


@implementer(IVocabularyFactory)
class FinanziatoriBandoVocabularyFactory(BandiBaseVocabularyFactory):

    vocab_name = FINANZIATORI_BANDO


@implementer(IVocabularyFactory)
class MaterieBandoVocabularyFactory(BandiBaseVocabularyFactory):

    vocab_name = MATERIE_BANDO


TipologieBandoVocabulary = TipologieBandoVocabularyFactory()
DestinatariBandoVocabulary = DestinatariBandoVocabularyFactory()
FinanziatoriBandoVocabulary = FinanziatoriBandoVocabularyFactory()
MaterieBandoVocabulary = MaterieBandoVocabularyFactory()



# -*- coding: utf-8 -*-
from plone import api
from plone.app.testing import login
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from rer.bandi.testing import RER_BANDI_FUNCTIONAL_TESTING

import unittest


class TestCollectionCriteria(unittest.TestCase):

    layer = RER_BANDI_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        login(self.portal, TEST_USER_NAME)
        self.collection = api.content.create(
            container=self.portal, type='Collection', title='Collection'
        )
        self.bando1 = api.content.create(
            container=self.portal,
            type='Bando',
            title='Bando with destinatari',
            destinatari=[u'd1', u'd2', u'dà'],
        )

        self.bando2 = api.content.create(
            container=self.portal,
            type='Bando',
            title='Bando with finanziatori',
            finanziatori=['f1'],
        )

        self.bando3 = api.content.create(
            container=self.portal,
            type='Bando',
            title='Bando with materie',
            materie=['m1', 'm2'],
        )

    def test_query_destinatari(self):
        query = [
            {
                'i': 'destinatari',
                'o': 'plone.app.querystring.operation.string.is',
                'v': ['d1'],
            }
        ]
        self.collection.setQuery(query)
        results = self.collection.results()
        self.assertEqual(results.sequence_length, 1)
        self.assertEqual(results[0].Title(), self.bando1.Title())

    def test_query_destinatari_with_accent(self):
        query = [
            {
                'i': 'destinatari',
                'o': 'plone.app.querystring.operation.string.is',
                'v': ['dà'],
            }
        ]
        self.collection.setQuery(query)
        results = self.collection.results()
        self.assertEqual(results.sequence_length, 1)
        self.assertEqual(results[0].Title(), self.bando1.Title())

    def test_query_finanziatori(self):
        query = [
            {
                'i': 'finanziatori',
                'o': 'plone.app.querystring.operation.string.is',
                'v': ['f1'],
            }
        ]
        self.collection.setQuery(query)
        results = self.collection.results()
        self.assertEqual(results.sequence_length, 1)
        self.assertEqual(results[0].Title(), self.bando2.Title())

    def test_query_materie(self):
        query = [
            {
                'i': 'materie',
                'o': 'plone.app.querystring.operation.string.is',
                'v': ['m1'],
            }
        ]
        self.collection.setQuery(query)
        results = self.collection.results()
        self.assertEqual(results.sequence_length, 1)
        self.assertEqual(results[0].Title(), self.bando3.Title())



# -*- coding: utf-8 -*-
from plone.app.vocabularies.catalog import KeywordsVocabulary


class TipologieBandoKeywordsVocabularyFactory(KeywordsVocabulary):

    keyword_index = 'tipologia_bando'


class MaterieBandoKeywordsVocabularyFactory(KeywordsVocabulary):

    keyword_index = 'materie'


class FinanziatoriBandoKeywordsVocabularyFactory(KeywordsVocabulary):

    keyword_index = 'finanziatori'


class DestinatariBandoKeywordsVocabularyFactory(KeywordsVocabulary):

    keyword_index = 'destinatari'


TipologieBandoKeywordsVocabulary = TipologieBandoKeywordsVocabularyFactory()
DestinatariBandoKeywordsVocabulary = (
    DestinatariBandoKeywordsVocabularyFactory()
)
FinanziatoriBandoKeywordsVocabulary = (
    FinanziatoriBandoKeywordsVocabularyFactory()
)
MaterieBandoKeywordsVocabulary = MaterieBandoKeywordsVocabularyFactory()



# -*- coding: utf-8 -*-
from plone.dexterity.content import Container
from rer.bandi.interfaces.bandofolderdeepening import IBandoFolderDeepening
from zope.interface import implementer


@implementer(IBandoFolderDeepening)
class BandoFolderDeepening(Container):
    """ """



# -*- coding: utf-8 -*-
from plone import api
from Products.Five import BrowserView
from rer.bandi import _
from zope.component import getUtility
from zope.i18n import translate
from zope.interface import implementer
from zope.interface import Interface
from zope.schema.interfaces import IVocabularyFactory


class ICollectionBandiView(Interface):
    pass


@implementer(ICollectionBandiView)
class CollectionBandiView(BrowserView):
    def __init__(self, context, request):
        self.context = context
        self.request = request
        self.voc_tipologia = getUtility(
            IVocabularyFactory, name='rer.bandi.tipologie.vocabulary'
        )(self.context)

    def getTipologiaTitle(self, key):
        """
        """
        try:
            value = self.voc_tipologia.getTermByToken(key)
            return value.title
        except LookupError:
            return key

    def isValidDeadline(self, date):
        """
        """
        if not date:
            return False
        if date.Date() == '2100/12/31':
            # a default date for bandi that don't have a defined deadline
            return False
        return True

    def getScadenzaDate(self, brain):
        date = brain.scadenza_bando
        long_format = True
        if brain.scadenza_bando.Time() == '00:00:00':
            # indexer add 1 day to this date, to make a bando ends at midnight
            # of the day-after, if time is not provided
            date = date - 1
            long_format = False
        return api.portal.get_localized_time(
            datetime=date, long_format=long_format
        )

    def getBandoState(self, bando):
        """
        return corretc bando state
        """
        scadenza_bando = bando.scadenza_bando
        chiusura_procedimento_bando = bando.chiusura_procedimento_bando
        state = ('open', translate(_(u'Open'), context=self.request))
        if scadenza_bando and scadenza_bando.isPast():
            if (
                chiusura_procedimento_bando
                and chiusura_procedimento_bando.isPast()
            ):
                state = (
                    'closed',
                    translate(_(u'Closed'), context=self.request),
                )
            else:
                state = (
                    'inProgress',
                    translate(_(u'In progress'), context=self.request),
                )
        else:
            if (
                chiusura_procedimento_bando
                and chiusura_procedimento_bando.isPast()
            ):
                state = (
                    'closed',
                    translate(_(u'Closed'), context=self.request),
                )
        return state



# -*- coding: utf-8 -*-
from DateTime import DateTime
from plone.restapi.search.handler import SearchHandler
from plone.restapi.search.utils import unflatten_dotted_dict
from plone.restapi.services.search.get import SearchGet
from zope.interface import implementer
from zope.publisher.interfaces import IPublishTraverse


@implementer(IPublishTraverse)
class SearchBandiGet(SearchGet):
    def __init__(self, context, request):
        super(SearchBandiGet, self).__init__(context, request)

    @property
    def query(self):
        query = self.request.form.copy()
        query = unflatten_dotted_dict(query)

        # Questi parametri vengono aggiunti di base a tutte le query
        base_query_parameters = {"portal_type": "Bando"}

        query.update(base_query_parameters)

        stato = query.get("stato_bandi")
        if stato:
            stato_query = self.query_stato(stato)
            del query['stato_bandi']
            query.update(stato_query)
        return query

    def query_stato(self, stato):
        """ In base allo stato che ci viene passato in ingresso generiamo
        la query corretta da passare poi al catalogo.

        Valori possibili per stato: [open, inProgress, closed]
        """

        query = {}

        if stato:
            now = DateTime()
            if stato == "open":
                query["scadenza_bando"] = {"query": now, "range": "min"}
                query["chiusura_procedimento_bando"] = {
                    "query": now,
                    "range": "min",
                }
            if stato == "inProgress":
                query["scadenza_bando"] = {"query": now, "range": "max"}
                query["chiusura_procedimento_bando"] = {
                    "query": now,
                    "range": "min",
                }
            if stato == "closed":
                query["chiusura_procedimento_bando"] = {
                    "query": now,
                    "range": "max",
                }
            return query
        else:
            return {}

    def reply(self):
        return SearchHandler(self.context, self.request).search(self.query)



# -*- coding: utf-8 -*-
from Products.CMFCore.utils import getToolByName
from plone import api
from rer.bandi import logger

default_profile = 'profile-rer.bandi:default'

TIPOLOGIA_BANDO_MAPPING = {
    'agevolazioni': u'Agevolazioni, finanziamenti, contributi',
    'beni_servizi': u'Manifestazioni di interesse',
    'lavori_pubblici': u'Manifestazioni di interesse',
    'altro': u'Manifestazioni di interesse',
}

DESTINATARI_BANDO_MAPPING = {
    'Cittadini': [u'Cittadini'],
    'Imprese': [u'Grandi imprese', u'PMI', u'Micro imprese'],
    'Enti locali': [u'Enti pubblici'],
    'Associazioni': [u'Enti del Terzo settore'],
    'Altro': [u'Scuole, università, enti di formazione'],
}


def remap_fields(brain):
    bando = brain.getObject()
    tipologia = getattr(bando, 'tipologia_bando', '')
    destinatari = getattr(bando, 'destinatari', [])

    if tipologia:
        if tipologia not in TIPOLOGIA_BANDO_MAPPING:
            logger.warning(
                '  - Unable to find a match for tipologia "{tipologia}" in "{bando}"'.format(  # noqa
                    tipologia=tipologia, bando=brain.getPath()
                )
            )
        else:
            new_value = TIPOLOGIA_BANDO_MAPPING[tipologia]
            logger.info(
                '  - TIPOLOGIA: {old} => {new}'.format(
                    old=tipologia, new=new_value
                )
            )
            bando.tipologia_bando = new_value.decode('utf-8')

    if not destinatari:
        new_value = DESTINATARI_BANDO_MAPPING['Altro']
        bando.destinatari = new_value
        logger.info('  - DESTINATARIO: VUOTO => {new}'.format(new=new_value))
    else:
        new_value = []
        for destinatario in destinatari:
            if destinatario not in DESTINATARI_BANDO_MAPPING:
                logger.warning(
                    '  - Unable to find a match for destinatario "{destinatario}" in "{bando}"'.format(  # noqa
                        destinatario=destinatario, bando=brain.getPath()
                    )
                )
            else:
                new_value.extend(DESTINATARI_BANDO_MAPPING[destinatario])
        if new_value:
            logger.info(
                '  - DESTINATARIO: {old} => {new}'.format(
                    old=destinatari, new=new_value
                )
            )
            bando.destinatari = new_value
    bando.reindexObject(idxs=['destinatari', 'tipologia_bando'])


def upgrade(upgrade_product, version):
    """ Decorator for updating the QuickInstaller of a upgrade """

    def wrap_func(fn):
        def wrap_func_args(context, *args):
            p = getToolByName(context, 'portal_quickinstaller').get(
                upgrade_product
            )
            setattr(p, 'installedversion', version)
            return fn(context, *args)

        return wrap_func_args

    return wrap_func


@upgrade('rer.bandi', '2.1.0')
def to_2(context):
    """
    """
    logger.info('Upgrading rer.bandi to version 2.1.0')
    setup_tool = getToolByName(context, 'portal_setup')
    setup_tool.runImportStepFromProfile(default_profile, 'catalog')


def migrate_to_2200(context):
    PROFILE_ID = 'profile-rer.bandi:migrate_to_2200'
    setup_tool = getToolByName(context, 'portal_setup')
    setup_tool.runAllImportStepsFromProfile(PROFILE_ID)
    setup_tool.runImportStepFromProfile(default_profile, 'catalog')
    logger.info("Reindexing catalog indexes")
    catalog = getToolByName(context, 'portal_catalog')
    bandi = catalog(portal_type="Bando")
    for bando in bandi:
        bando.getObject().reindexObject(
            idxs=[
                "chiusura_procedimento_bando",
                "destinatari",
                "scadenza_bando",
                "tipologia_bando",
            ]
        )

    setup_tool.runImportStepFromProfile(
        'profile-rer.bandi:default', 'plone.app.registry'
    )
    setup_tool.runImportStepFromProfile(
        'profile-rer.bandi:default', 'typeinfo'
    )

    logger.info("Migrated to 2.2.0")


def migrate_to_2300(context):
    setup_tool = api.portal.get_tool('portal_setup')
    setup_tool.runImportStepFromProfile(default_profile, 'plone.app.registry')
    logger.info('Add sortable collection criteria')


def migrate_to_2400(context):
    setup_tool = api.portal.get_tool('portal_setup')
    setup_tool.runImportStepFromProfile(default_profile, 'typeinfo')
    logger.info('Upgrading to 2400')


def migrate_to_2500(context):
    setup_tool = api.portal.get_tool('portal_setup')
    setup_tool.runImportStepFromProfile(default_profile, 'typeinfo')
    logger.info('Upgrading to 2500')


def migrate_to_3000(context):
    PROFILE_ID = 'profile-rer.bandi:migrate_to_3000'
    setup_tool = getToolByName(context, 'portal_setup')
    setup_tool.runAllImportStepsFromProfile(PROFILE_ID)

    #  update indexes and topics
    setup_tool.runImportStepFromProfile(default_profile, 'catalog')
    setup_tool.runImportStepFromProfile(default_profile, 'plone.app.registry')

    bandi = api.content.find(portal_type='Bando')
    tot_results = len(bandi)
    logger.info('### There are {tot} Bandi to fix ###'.format(tot=tot_results))
    for counter, brain in enumerate(bandi):
        logger.info(
            '[{counter}/{tot}] - {bando}'.format(
                counter=counter + 1, tot=tot_results, bando=brain.getPath()
            )
        )
        remap_fields(brain=brain)
    logger.info('Upgrading to 3000')


def migrate_to_3100(context):
    PROFILE_ID = 'profile-rer.bandi:migrate_to_3100'
    setup_tool = getToolByName(context, 'portal_setup')
    setup_tool.runAllImportStepsFromProfile(PROFILE_ID)

    #  update indexes and topics
    setup_tool.runImportStepFromProfile(default_profile, 'catalog')
    setup_tool.runImportStepFromProfile(default_profile, 'plone.app.registry')

    bandi = api.content.find(portal_type='Bando')
    tot_results = len(bandi)
    logger.info('### Fixing {tot} Bandi ###'.format(tot=tot_results))
    for counter, brain in enumerate(bandi):
        logger.info(
            '[{counter}/{tot}] - {bando}'.format(
                counter=counter + 1, tot=tot_results, bando=brain.getPath()
            )
        )
        bando = brain.getObject()
        bando.reindexObject(idxs=[
            "chiusura_procedimento_bando",
            "destinatari",
            "scadenza_bando",
            "tipologia_bando",
            "finanziatori",
            "materie",
        ])

    criteria_mapping = {
        u'getTipologia_bando': u'tipologia_bando',
        u'getChiusura_procedimento_bando': u'chiusura_procedimento_bando',
        u'getScadenza_bando': u'scadenza_bando',
        u'getFinanziatori_bando': u'finanziatori',
        u'getMaterie_bando': u'materie',
        u'getDestinatariBando': u'destinatari'
    }
    collections = api.content.find(portal_type='Collection')
    tot_results = len(collections)
    logger.info('### Fixing {tot} Collections ###'.format(tot=tot_results))
    for counter, brain in enumerate(collections):
        collection = brain.getObject()
        query = []
        for criteria in getattr(collection, 'query', []):
            criteria['i'] = criteria_mapping.get(criteria['i'], criteria['i'])
            query.append(criteria)
        collection.query = query
        logger.info(
            '[{counter}/{tot}] - {collection}'.format(
                counter=counter + 1, tot=tot_results, collection=brain.getPath()
            )
        )
    logger.info('Upgrade to 3100 complete')



# -*- coding: utf-8 -*-
from plone.restapi.services import Service
from zope.component import getUtility
from zope.i18n import translate
from zope.interface import implementer
from zope.publisher.interfaces import IPublishTraverse
from zope.schema.interfaces import IVocabularyFactory
from rer.bandi import _


@implementer(IPublishTraverse)
class SearchParametersGet(Service):
    def __init__(self, context, request):
        super(SearchParametersGet, self).__init__(context, request)

    def reply(self):
        return [
            {
                'id': 'SearchableText',
                'label': translate(
                    _('bandi_search_text_label', default=u'Search text'),
                    context=self.request,
                ),
                'help': '',
                'type': 'text',
            },
            {
                'id': 'stato_bandi',
                'label': translate(
                    _('bandi_search_state_label', default=u'State'),
                    context=self.request,
                ),
                'help': '',
                'type': 'select',
                'multivalued': False,
                'options': [
                    {
                        'label': translate(
                            _('bando_state_all_select_label', default='All'),
                            context=self.request,
                        ),
                        'value': '',
                    },
                    {
                        'label': translate(
                            _('bando_state_open_select_label', default='Open'),
                            context=self.request,
                        ),
                        'value': 'open',
                    },
                    {
                        'label': translate(
                            _(
                                'bando_state_inProgress_select_label',
                                default='In progress',
                            ),
                            context=self.request,
                        ),
                        'value': 'inProgress',
                    },
                    {
                        'label': translate(
                            _(
                                'bando_state_closed_select_label',
                                default='Closed',
                            ),
                            context=self.request,
                        ),
                        'value': 'closed',
                    },
                ],
            },
            {
                'id': 'tipologia_bando',
                'label': translate(
                    _('bandi_search_type_label', default='Type'),
                    context=self.request,
                ),
                'help': '',
                'type': 'checkbox',
                'options': self.getVocabularyTermsForForm(
                    'rer.bandi.tipologie.vocabulary'
                ),
            },
            {
                'id': 'destinatari',
                'label': translate(
                    _('destinatari_label', default='Who can apply'),
                    context=self.request,
                ),
                'help': '',
                'type': 'select',
                'multivalued': True,
                'options': self.getVocabularyTermsForForm(
                    'rer.bandi.destinatari.vocabulary'
                ),
            },
            {
                'id': 'finanziatori',
                'label': translate(
                    _(
                        'finanziatori_label',
                        default='Financed by EU programmes',
                    ),
                    context=self.request,
                ),
                'help': '',
                'type': 'select',
                'multivalued': True,
                'options': self.getVocabularyTermsForForm(
                    'rer.bandi.finanziatori.vocabulary'
                ),
            },
            {
                'id': 'materie',
                'label': translate(
                    _('materie_label', default='Topic'), context=self.request
                ),
                'help': '',
                'type': 'select',
                'multivalued': True,
                'options': self.getVocabularyTermsForForm(
                    'rer.bandi.materie.vocabulary'
                ),
            },
            {
                'id': 'Subject',
                'label': translate(
                    _('subject_label', default='Subjects'),
                    context=self.request,
                ),
                'help': '',
                'type': 'select',
                'multivalued': True,
                'options': self.getVocabularyTermsForForm(
                    'plone.app.vocabularies.Keywords'
                ),
            },
        ]

    def getVocabularyTermsForForm(self, vocab_name):
        """
        Return the values of vocabulary
        """
        utility = getUtility(IVocabularyFactory, vocab_name)

        values = []

        vocab = utility(self.context)

        for entry in vocab:
            if entry.title != u'select_label':
                values.append({'value': entry.value, 'label': entry.title})
        return values



# -*- coding: utf-8 -*-
"""
This module contains the tool of rer.bandi
"""
import os
from setuptools import setup, find_packages

version = '4.1.0'

setup(
    name='rer.bandi',
    version=version,
    description="A product for announcements management",
    long_description=open("README.rst").read()
    + "\n"
    + open(os.path.join("docs", "HISTORY.txt")).read(),
    # Get more strings from
    # http://pypi.python.org/pypi?:action=list_classifiers
    classifiers=[
        "Environment :: Web Environment",
        "Framework :: Plone :: 5.1",
        "Framework :: Plone :: 5.2",
        "Framework :: Plone :: Addon",
        "Framework :: Plone",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python",
    ],
    keywords='rer bandi announcements',
    author='RedTurtle Technology',
    author_email='sviluppoplone@redturtle.it',
    url='https://github.com/PloneGov-IT/rer.bandi',
    project_urls={
        'PyPI': 'https://pypi.python.org/pypi/rer.bandi',
        'Source': 'https://github.com/PloneGov-IT/rer.bandi',
        'Tracker': 'https://github.com/PloneGov-IT/rer.bandi/issues',
        # 'Documentation': 'https://rer.bandi.readthedocs.io/en/latest/',
    },
    license='GPL',
    packages=find_packages(exclude=['ez_setup']),
    namespace_packages=['rer'],
    include_package_data=True,
    zip_safe=False,
    install_requires=['setuptools', 'lxml', 'plone.restapi'],
    extras_require={
        'test': [
            'plone.app.testing',
            # Plone KGS does not use this version, because it would break
            # Remove if your package shall be part of coredev.
            # plone_coredev tests as of 2016-04-01.
            'plone.testing>=5.0.0',
            'plone.app.contenttypes',
            'plone.app.robotframework[debug]',
            'requests',
        ]
    },
    test_suite='rer.bandi.tests.test_docs.test_suite',
    entry_points="""
      # -*- entry_points -*-
      [z3c.autoinclude.plugin]
      target = plone
      """,
)



# -*- coding: utf-8 -*-
from datetime import datetime
from DateTime import DateTime
from plone.indexer.decorator import indexer
from rer.bandi.interfaces.bando import IBando

import six


def dateToDatetime(d):
    return datetime.combine(d, datetime.min.time())


@indexer(IBando)
def destinatari_bando(object, **kw):
    destinatari = getattr(object, 'destinatari', [])
    if not destinatari:
        return []
    if six.PY2:
        return [x.encode('utf-8') for x in destinatari]
    return destinatari


@indexer(IBando)
def chiusura_procedimento_bando(object, **kw):

    date_chiusura_procedimento_bando = getattr(
        object, 'chiusura_procedimento_bando', None
    )
    if date_chiusura_procedimento_bando:
        datetime_chiusura_procedimento_bando = dateToDatetime(
            date_chiusura_procedimento_bando
        )
    else:
        return DateTime('2100/12/31')

    if datetime_chiusura_procedimento_bando:
        return DateTime(datetime_chiusura_procedimento_bando)


@indexer(IBando)
def scadenza_bando(object, **kw):
    datetime_scadenza_bando = getattr(object, 'scadenza_bando', None)
    if not datetime_scadenza_bando:
        return DateTime('2100/12/31')
    zope_dt_scadenza_bando = DateTime(datetime_scadenza_bando)
    if zope_dt_scadenza_bando.Time() == '00:00:00':
        return zope_dt_scadenza_bando + 1
    else:
        return zope_dt_scadenza_bando


@indexer(IBando)
def tipologia_bando(object, **kw):
    tipologia = getattr(object, 'tipologia_bando', '')
    if six.PY2:
        return tipologia.encode('utf-8')
    return tipologia


@indexer(IBando)
def finanziatori(object, **kw):
    finanziatori = getattr(object, 'finanziatori', [])
    if not finanziatori:
        return []
    if six.PY2:
        return [x.encode('utf-8') for x in finanziatori]
    return finanziatori


@indexer(IBando)
def materie(object, **kw):
    materie = getattr(object, 'materie', [])
    if not materie:
        return []
    if six.PY2:
        return [x.encode('utf-8') for x in materie]
    return materie



# -*- extra stuff goes here -*-
from rer.bandi.interfaces.bando import IBando
from rer.bandi.interfaces.bandofolderdeepening import IBandoFolderDeepening
from rer.bandi.interfaces.browserlayer import IRerBandiLayer

IBando
IBandoFolderDeepening
IRerBandiLayer



from zope.interface import Interface
from zope import schema


class IBandoSettings(Interface):
    """
    DEPRECATED: It's still here only for 3000 upgrade-step
    """

    default_ente = schema.Tuple(
        title=u"default_ente",
        required=False,
        value_type=schema.TextLine(),
        missing_value=None,
    )

    default_destinatari = schema.Tuple(
        title=u"default_destinatari_bandi",
        required=False,
        value_type=schema.TextLine(),
        missing_value=None,
    )
